"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createMenuCrud = void 0;
var _prepareMenuItems = _interopRequireDefault(require("./menus/prepareMenuItems"));
var _error = _interopRequireDefault(require("@webiny/error"));
var _handlerGraphql = require("@webiny/handler-graphql");
var _pubsub = require("@webiny/pubsub");
var _validation = require("./menus/validation");
var _utils = require("@webiny/utils");
const createMenuCrud = params => {
  const {
    context,
    storageOperations,
    getLocaleCode,
    getTenantId,
    menusPermissions
  } = params;

  // create
  const onMenuBeforeCreate = (0, _pubsub.createTopic)("pageBuilder.onMenuBeforeCreate");
  const onMenuAfterCreate = (0, _pubsub.createTopic)("pageBuilder.onMenuAfterCreate");
  // update
  const onMenuBeforeUpdate = (0, _pubsub.createTopic)("pageBuilder.onMenuBeforeUpdate");
  const onMenuAfterUpdate = (0, _pubsub.createTopic)("pageBuilder.onMenuAfterUpdate");
  // delete
  const onMenuBeforeDelete = (0, _pubsub.createTopic)("pageBuilder.onMenuBeforeDelete");
  const onMenuAfterDelete = (0, _pubsub.createTopic)("pageBuilder.onMenuAfterDelete");
  return {
    onMenuBeforeCreate,
    onMenuAfterCreate,
    onMenuBeforeUpdate,
    onMenuAfterUpdate,
    onMenuBeforeDelete,
    onMenuAfterDelete,
    async getMenu(slug, options) {
      const {
        auth = true
      } = options || {};
      if (auth !== false) {
        await menusPermissions.ensure({
          rwd: "r"
        });
      }
      const params = {
        where: {
          slug,
          tenant: getTenantId(),
          locale: getLocaleCode()
        }
      };
      let menu;
      try {
        menu = await storageOperations.menus.get(params);
      } catch (ex) {
        throw new _error.default(ex.message || "Could not get menu by slug.", ex.code || "GET_MENU_ERROR", {
          ...(ex.data || {}),
          params
        });
      }
      if (!menu) {
        return null;
      }
      if (auth !== false) {
        await menusPermissions.ensure({
          owns: menu.createdBy
        });
      }
      return menu;
    },
    /**
     * Used to fetch menu data from a public website. Items are prepared for consumption too.
     * @param slug
     */
    async getPublicMenu(slug) {
      const menu = await this.getMenu(slug, {
        auth: false
      });
      if (!menu) {
        throw new _handlerGraphql.NotFoundError();
      }
      menu.items = await (0, _prepareMenuItems.default)({
        menu,
        context
      });
      return menu;
    },
    async listMenus(params) {
      await menusPermissions.ensure({
        rwd: "r"
      });
      const {
        sort
      } = params || {};
      const listParams = {
        where: {
          tenant: getTenantId(),
          locale: getLocaleCode()
        },
        sort: Array.isArray(sort) && sort.length > 0 ? sort : ["createdOn_ASC"]
      };

      // If user can only manage own records, let's add that to the listing.
      if (await menusPermissions.canAccessOnlyOwnRecords()) {
        const identity = context.security.getIdentity();
        listParams.where.createdBy = identity.id;
      }
      try {
        const [items] = await storageOperations.menus.list(listParams);
        return items;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not list all menus.", ex.code || "LIST_MENUS_ERROR", {
          params
        });
      }
    },
    async createMenu(input) {
      await menusPermissions.ensure({
        rwd: "w"
      });
      const validationResult = await (0, _validation.createMenuCreateValidation)().safeParseAsync(input);
      if (!validationResult.success) {
        throw (0, _utils.createZodError)(validationResult.error);
      }
      const data = validationResult.data;
      const existing = await storageOperations.menus.get({
        where: {
          slug: data.slug,
          tenant: getTenantId(),
          locale: getLocaleCode()
        }
      });
      if (existing) {
        throw new _error.default(`Menu "${data.slug}" already exists.`);
      }
      const identity = context.security.getIdentity();
      const menu = {
        ...data,
        items: data.items || [],
        createdOn: new Date().toISOString(),
        createdBy: {
          id: identity.id,
          type: identity.type,
          displayName: identity.displayName
        },
        tenant: getTenantId(),
        locale: getLocaleCode()
      };
      try {
        await onMenuBeforeCreate.publish({
          input: data,
          menu
        });
        const result = await storageOperations.menus.create({
          input: data,
          menu
        });
        await onMenuAfterCreate.publish({
          input: data,
          menu: result
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not create menu.", ex.code || "CREATE_MENU_ERROR", {
          ...(ex.data || {}),
          menu
        });
      }
    },
    async updateMenu(slug, input) {
      await menusPermissions.ensure({
        rwd: "w"
      });
      const original = await this.getMenu(slug);
      if (!original) {
        throw new _handlerGraphql.NotFoundError(`Menu "${slug}" not found.`);
      }
      await menusPermissions.ensure({
        owns: original.createdBy
      });
      const validationResult = await (0, _validation.createMenuUpdateValidation)().safeParseAsync(input);
      if (!validationResult.success) {
        throw (0, _utils.createZodError)(validationResult.error);
      }
      const data = (0, _utils.removeUndefinedValues)(validationResult.data);
      const menu = {
        ...original,
        ...data
      };
      try {
        await onMenuBeforeUpdate.publish({
          original,
          menu
        });
        const result = await storageOperations.menus.update({
          input: data,
          original,
          menu
        });
        await onMenuAfterUpdate.publish({
          original,
          menu: result
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not update menu.", ex.code || "UPDATE_MENU_ERROR", {
          ...(ex.data || {}),
          original,
          menu
        });
      }
    },
    async deleteMenu(slug) {
      await menusPermissions.ensure({
        rwd: "d"
      });
      const menu = await this.getMenu(slug);
      if (!menu) {
        throw new _handlerGraphql.NotFoundError(`Menu "${slug}" not found.`);
      }
      await menusPermissions.ensure({
        owns: menu.createdBy
      });
      try {
        await onMenuBeforeDelete.publish({
          menu
        });
        const result = await storageOperations.menus.delete({
          menu
        });
        await onMenuAfterDelete.publish({
          menu: result
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not delete menu.", ex.code || "DELETE_MENU_ERROR", {
          ...(ex.data || {}),
          menu
        });
      }
    }
  };
};
exports.createMenuCrud = createMenuCrud;

//# sourceMappingURL=menus.crud.js.map